package asyncservice;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.TimeUnit;

class ExampleService {

    private static final Logger LOGGER = LoggerFactory.getLogger(Main.class);

    String work() {
        sleep(7000, TimeUnit.MILLISECONDS); /* Pretend to be busy... */
        char[] str = new char[5];
        ThreadLocalRandom current = ThreadLocalRandom.current();
        for (int idx = 0; idx < str.length; ++idx)
            str[idx] = (char) ('A' + current.nextInt(26));
        String msg = new String(str);
        log("Generated message: " + msg);
        return msg;
    }

    private void sleep(long average, TimeUnit unit) {
        String name = Thread.currentThread().getName();
        long timeout = Math.min(exponential(average), Math.multiplyExact(10, average));
        log("{} sleeping {} {}...", name, timeout, unit);
        try {
            unit.sleep(timeout);
            log(name + " awoke.");
        } catch (InterruptedException abort) {
            Thread.currentThread().interrupt();
            log(name + " interrupted.");
        }
    }

    private long exponential(long avg) {
        return (long) (avg * -Math.log(1 - ThreadLocalRandom.current().nextDouble()));
    }

    private void log(String s, Object... o) {
        LOGGER.info(s, o);
    }
}
