package asyncservice;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.*;


public class Main {

    private static final Logger LOGGER = LoggerFactory.getLogger(Main.class);

    private static <T> Callable<T> lazyval(T val, long timeout) {
        return () -> {
            Thread.sleep(1000);
            log("Task completed with : " + val);
            return val;
        };
    }

    public static void main(String[] args) throws InterruptedException {
        ExecutorService executor = Executors.newFixedThreadPool(4);
        final Future<Integer> asyncRes = executor.submit(lazyval(1, 100L));
        final Future<String> asyncRes1 = executor.submit(lazyval("Hello", 20L));
        final Future<String> asyncRes2 = executor.submit(lazyval("World", 100L));
        Thread.sleep(15000);
        log("End of executor example.");
        executor.shutdown();
        getTaskNotificationWithoutBlocking();
    }

    private static void getTaskNotificationWithoutBlocking() throws InterruptedException {
        ExampleService svc = new ExampleService();
        CompletableFuture<String> future = CompletableFuture.supplyAsync(svc::work);
        Main listener = new Main();
        future.thenAccept(listener::notify);
        Thread.sleep(10000);
        log("Exciting main...");
    }

    private void notify(String s) {
        log("Received message ", s);
    }

    private static void log(String s, Object... o) {
        LOGGER.info(s, o);
    }

}
