package ovh.gasser.translation;

import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.*;
import java.util.Arrays;

final class InjectionUtils {
    private static final XPath xPath;
    private static final Document document;

    static {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            var resource = InjectionUtils.class.getResource("/translations.xml");
            DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
            document = documentBuilder.parse(resource.getFile());
            XPathFactory xpf = XPathFactory.newInstance();
            xPath = xpf.newXPath();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Private ctor to prevent instantiation
     */
    private InjectionUtils() {
        throw new IllegalStateException();
    }

    static void populateTranslations(Class<?> clazz, String language) {
        Arrays.stream(clazz.getDeclaredFields())
                .filter(field -> field.isAnnotationPresent(InjectTranslation.class))
                .forEach(field -> {
                    try {
                        String expression = String.format("/translations/string[@key='%s']/translation[@lang='%s']", field.getName(), language);
                        XPathExpression xPathExpression = xPath.compile(expression);
                        String s = (String) xPathExpression.evaluate(document, XPathConstants.STRING);
                        field.set(null, s);
                    } catch (IllegalAccessException e) {
                        throw new RuntimeException("Unable to setup constant for field " + field.getName(), e);
                    } catch (XPathExpressionException e) {
                        throw new RuntimeException(e);
                    }
                });
    }
}
