package fr.gasser.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

public class Mandelbrot extends JFrame {

    private static final int imgSize = 256;

    private final BufferedImage img;

    private Mandelbrot(double xc, double yc, double size) throws HeadlessException {
        super("fr.gasser.gui.Mandelbrot set");
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        int max = 255;   // maximum number of iterations

        setPreferredSize(new Dimension(imgSize, imgSize));
        setMinimumSize(getPreferredSize());
        setMaximumSize(getPreferredSize());
        setResizable(false);


        img = new BufferedImage(imgSize, imgSize, BufferedImage.TYPE_INT_RGB);
        for (int i = 0; i < imgSize; i++) {
            for (int j = 0; j < imgSize; j++) {
                double x0 = xc - size/2 + size*i/imgSize;
                double y0 = yc - size/2 + size*j/imgSize;
                Complex z0 = new Complex(x0, y0);
                int gray = max - mand(z0, max);
                Color color = new Color(gray, gray, gray);
                img.setRGB(i, imgSize-1-j, color.getRGB());
            }
        }

        setVisible(true);
    }

    // return number of iterations to check if c = a + ib is in fr.gasser.gui.Mandelbrot set
    private int mand(Complex z0, int max) {
        Complex z = z0;
        for (int t = 0; t < max; t++) {
            if (z.abs() > 2.0) return t;
            z = z.times(z).plus(z0);
        }
        return max;
    }

    @Override
    public void paint(Graphics g) {
        g.drawImage(img, 0, 0, this);
    }

    private static class Complex {

        private final double re;
        private final double im;

        Complex(double real, double imaginary) {
            this.re = real;
            this.im = imaginary;
        }

        double abs() {
            return re * re + im * im;
        }

        Complex times(Complex b) {
            Complex a = this;
            double real = a.re * b.re - a.im * b.im;
            double imag = a.re * b.im + a.im * b.re;
            return new Complex(real, imag);
        }

        Complex plus(Complex other) {
            return new Complex(this.re + other.re, this.im + other.im);
        }
    }

    public static void main(String[] args)  {

        EventQueue.invokeLater(() -> new Mandelbrot(-1, -0.25, 0.25));
    }
}