package fr.gasser.gui.editor;

import javax.swing.*;
import java.awt.*;
import java.util.Arrays;
import java.util.Comparator;

class FontChooser extends JComboBox<Font> {

    FontChooser() {

        final Font[] fonts = GraphicsEnvironment
                .getLocalGraphicsEnvironment()
                .getAllFonts();

        Arrays.sort(fonts, Comparator.comparing(Font::getName));

        for (Font font : fonts) {
            if (font.canDisplayUpTo(font.getName()) == -1) {
                addItem(font);
            }
        }

        setRenderer(new FontCellRenderer());
    }

    Font get(Font font) {
        for (int i = 0; i < getModel().getSize(); i++) {
            final Font f = getItemAt(i);
            if (f.getName().equals(font.getName())) return f;
        }
        return null;
    }

    private static class FontCellRenderer
            implements ListCellRenderer<Font> {

        DefaultListCellRenderer renderer = new DefaultListCellRenderer();

        public Component getListCellRendererComponent(
                JList<? extends Font> list, Font font, int index,
                boolean isSelected, boolean cellHasFocus) {

            final Component result = renderer.getListCellRendererComponent(
                    list, font.getName(), index, isSelected, cellHasFocus);

            setFontPreserveSize(result, font);
            return result;
        }
    }

    private static void setFontPreserveSize(final Component comp, Font font) {
        final float size = comp.getFont().getSize();
        comp.setFont(font.deriveFont(size));
    }
}