package annotations.jsonserializer;

import annotations.JsonField;

import java.lang.reflect.Field;
import java.util.HashSet;
import java.util.Set;

class SerializableFieldExtractor {

    Set<SerializableField> getSerializableFields(Object o) {
        final Class<?> objectClass = o.getClass();
        final Set<SerializableField> jsonElements = new HashSet<>();
        try {
            for (Field field : objectClass.getDeclaredFields()) {
                field.setAccessible(true);
                if (field.isAnnotationPresent(JsonField.class)) {
                    jsonElements.add(new SerializableField(getSerializableField(field), (String) field.get(o)));
                }

            }
            return jsonElements;
        } catch (SecurityException | IllegalArgumentException | IllegalAccessException e) {
            throw new JsonSerializeException("JSON serializing error", e);
        }
    }

    private String getSerializableField(Field field) {
        final String annotationValue = field.getAnnotation(JsonField.class).value();
        return annotationValue.isEmpty() ? field.getName() : annotationValue;
    }
}