package patternmatching;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.function.Predicate;

@FunctionalInterface
public interface PatternMatcher<T, R> {

    static <T, R> PatternMatcher<T, R> when(Predicate<T> predicate, Function<T, R> action) {
        Objects.requireNonNull(predicate);
        Objects.requireNonNull(action);
        return value -> predicate.test(value) ? Optional.of(action.apply(value)) : Optional.empty();
    }

    default PatternMatcher<T, R> orWhen(Predicate<T> predicate, Function<T, R> action) {
        Objects.requireNonNull(predicate);
        Objects.requireNonNull(action);
        return value -> {
            final Optional<R> result = matches(value);
            if (result.isPresent()) return result;
            if (predicate.test(value)) return Optional.of(action.apply(value));
            return Optional.empty();
        };
    }

    default PatternMatcher<T, R> otherwise(Function<T, R> action) {
        Objects.requireNonNull(action);
        return value -> {
            final Optional<R> result = matches(value);
            return result.isPresent() ? result : Optional.of(action.apply(value));
        };
    }

    Optional<R> matches(T value);
}
