package annotations.jsonserializer;

import annotations.JsonField;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Set;
import java.util.stream.Collectors;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.containsInAnyOrder;
import static org.hamcrest.Matchers.containsString;

public class JsonSerializerTest {

    private final Logger logger = LoggerFactory.getLogger(getClass().getCanonicalName());

    @Test(expected = NullPointerException.class)
    public void serializeNullObjectEnsureNullPointerExceptionThrown() {
        new JsonSerializer(new SerializableFieldExtractor()).serialize(null);
    }

    @Test
    public void testSerialize() {
        final Car car = new Car();
        JsonSerializer serializer = new JsonSerializer(new SerializableFieldExtractor());
        final Set<SerializableField> jsonCar = serializer.serialize(car);
        final Set<String> names = jsonCar.stream().map(SerializableField::getName).collect(Collectors.toSet());
        final Set<String> values = jsonCar.stream().map(SerializableField::getValue).collect(Collectors.toSet());
        assertThat(names, containsInAnyOrder("model", "manufacturer"));
        assertThat(values, containsInAnyOrder("Ford", "F150"));
        logger.info(serializer.toJsonString(jsonCar));
    }

    @Test
    public void testSerializeToString() {
        final Car car = new Car();
        JsonSerializer serializer = new JsonSerializer(new SerializableFieldExtractor());
        final String jsonString = serializer.serializeToString(car);
        assertThat(jsonString, containsString("\"manufacturer\":\"Ford\""));
        assertThat(jsonString, containsString("\"model\":\"F150\""));
    }

    private static class Car {
        @JsonField("manufacturer")
        private final String make;
        @JsonField
        private final String model;
        private final String year;

        Car() {
            this.make = "Ford";
            this.model = "F150";
            this.year = "2018";
        }

        @Override
        public String toString() {
            return year + " " + make + " " + model;
        }
    }

}