package patternmatching;

import org.junit.Test;

import java.util.Optional;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;

public class PatternMatcherTest {

    @Test
    public void integerMatching() {
        final PatternMatcher<? super Number, String> pm = PatternMatcher
                .when(Number.class::isInstance, x -> "Integer: " + x);
        assertEquals(pm.matches(42), Optional.of("Integer: 42"));
    }

    @Test
    public void stringMatching() {
        final PatternMatcher<String, String> pm = PatternMatcher
                .when("world"::equals, x -> "Hello, " + x);
        assertThat(pm.matches("world"), is(Optional.of("Hello, world")));
        assertThat(pm.matches("Bob"), is(Optional.empty()));
    }

    @Test
    public void emptyMatching() {
        final PatternMatcher<? super Number, Optional<String>> pm = PatternMatcher
                .when(Integer.class::isInstance, x -> Optional.empty());
        assertEquals(pm.matches(42), Optional.of(Optional.empty()));
    }

    @Test
    public void testOrWhen() {
        final PatternMatcher<Object, String> pm = PatternMatcher
                .when("world"::equals, x -> "Hello, " + x)
                .orWhen(Double.class::isInstance, x -> "Double: " + x);
        assertThat(pm.matches("world"), is(Optional.of("Hello, world")));
        assertThat(pm.matches(1.42), is(Optional.of("Double: 1.42")));
    }

    @Test
    public void testOtherwise() {
        final PatternMatcher<Object, String> pm = PatternMatcher.when("world"::equals, x -> "Hello, " + x)
                .otherwise(x -> "got this object: " + x);
        assertThat(pm.matches("world"), is(Optional.of("Hello, world")));
        assertThat(pm.matches("foo"), is(Optional.of("got this object: foo")));
    }

    @Test
    public void testCombine() {
        final PatternMatcher<Object, String> pm = PatternMatcher
                .when("world"::equals, x -> "Hello, " + x)
                .orWhen(Double.class::isInstance, x -> "Double: " + x)
                .otherwise(x -> "got this object: " + x);
        assertThat(pm.matches("world"), is(Optional.of("Hello, world")));
        assertThat(pm.matches(1.42), is(Optional.of("Double: 1.42")));
        assertThat(pm.matches("foo"), is(Optional.of("got this object: foo")));
    }
}