package messagebroker;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.PrintWriter;

public abstract class BaseMessageHandler implements MessageHandler {
    private static final Logger LOGGER = LoggerFactory.getLogger(BaseMessageHandler.class);

    private MessageHandler nextHandler;
    private final MessageHandlerContext context;

    protected BaseMessageHandler(MessageHandlerContext context) {
        this.context = context;
        this.nextHandler = null;
    }

    public void handleMessage(String message) throws UnknownMessageFormatException {
        if (nextHandler != null) {
            LOGGER.debug("Trying next handler in chain: {}", nextHandler);
            nextHandler.handleMessage(message);
        } else {
            String msg = "Unable to handle message " + message;
            LOGGER.warn(msg);
            throw new UnknownMessageFormatException(msg);
        }
    }

    @Override
    public void setNextHandler(MessageHandler nextHandler) {
        this.nextHandler = nextHandler;
        LOGGER.debug("Next handler of {} set to {}", getClass().getCanonicalName(), nextHandler.getClass().getCanonicalName());
    }

    public MessageHandler getNextHandler() {
        return nextHandler;
    }

    protected MessageHandlerContext getContext() {
        return this.context;
    }

    protected void sendResponse(String response) {
        try (PrintWriter writer = new PrintWriter(getContext().getOutputStream())) {
            writer.println(response);
            writer.flush();
        }
    }
}
