package messagebroker;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

class MessageHandlerFactory {
    private static final Logger LOGGER = LoggerFactory.getLogger(MessageHandlerFactory.class);

    private final MessageHandlerContext context;

    private MessageHandler firstHandler = null;
    private MessageHandler lastHandler = null;

    MessageHandlerFactory(MessageHandlerContext context) {
        this.context = context;
    }

    void addHandler(String handlerName)
            throws ReflectiveOperationException {
        Class<? extends MessageHandler> handlerClass = Class.forName(handlerName)
                .asSubclass(MessageHandler.class);

        LOGGER.debug("Class {} found for handler name {}", handlerClass.getCanonicalName(), handlerName);
        MessageHandler next = newMessageHandler(handlerClass);
        if (firstHandler == null) {
            firstHandler = next;
            lastHandler = firstHandler;
        } else {
            lastHandler.setNextHandler(next);
            lastHandler = next;
        }
    }

    MessageHandler getHandler() {
        return firstHandler;
    }

    private MessageHandler newMessageHandler(Class<? extends MessageHandler> handlerClass)
            throws ReflectiveOperationException {
        var ctor = handlerClass.getConstructor(MessageHandlerContext.class);
        return ctor.newInstance(context);
    }
}
