package messagebroker.handlers;

import messagebroker.BaseMessageHandler;
import messagebroker.MessageHandlerContext;
import messagebroker.UnknownMessageFormatException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.regex.Pattern;

public class EchoMessageHandler extends BaseMessageHandler {
    private static final Logger LOGGER = LoggerFactory.getLogger(EchoMessageHandler.class);
    private static final Pattern PATTERN = Pattern.compile("ECHO (.*)$");

    public EchoMessageHandler(MessageHandlerContext context) {
        super(context);
    }

    @Override
    public void handleMessage(String message) throws UnknownMessageFormatException {
        var matcher = PATTERN.matcher(message);
        if (matcher.matches()) {
            String parsed = matcher.group(1);
            LOGGER.info("Parsed message {}", parsed);
            sendResponse(parsed);
        } else {
            LOGGER.debug("Pattern  \"{}\" does not match. Cannot handle message {}", PATTERN, message);
            super.handleMessage(message);
        }
    }
}
