package messagebroker.handlers;

import messagebroker.BaseMessageHandler;
import messagebroker.MessageHandlerContext;
import messagebroker.UnknownMessageFormatException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ListHandlersMessageHandler extends BaseMessageHandler {
    private static final Logger LOGGER = LoggerFactory.getLogger(EchoMessageHandler.class);
    private static final String PATTERN = "CONTROL.LIST_HANDLERS";

    public ListHandlersMessageHandler(MessageHandlerContext context) {
        super(context);
    }

    @Override
    public void handleMessage(String message) throws UnknownMessageFormatException {
        if (message.equals(PATTERN)) {
            LOGGER.info("Received message {}", message);
            var response = new StringBuilder();
            for (var nextHandler = getContext().getBroker().getHandler(); nextHandler != null;  nextHandler = nextHandler.getNextHandler()){
                response.append(nextHandler.getClass().getCanonicalName())
                        .append("\n");
            }
            sendResponse(response.toString());
        } else {
            LOGGER.debug("Pattern \"{}\" does not match. Cannot handle message {}", PATTERN, message);
            super.handleMessage(message);
        }
    }
}
