package messagebroker.handlers;

import messagebroker.MessageBroker;
import messagebroker.MessageHandlerContext;
import messagebroker.UnknownMessageFormatException;
import org.junit.Test;

import java.io.OutputStream;

import static org.mockito.Mockito.*;

public class AddHandlerMessageHandlerTest {

    private MessageHandlerContext context = mock(MessageHandlerContext.class);

    public AddHandlerMessageHandlerTest() {
        var broker = mock(MessageBroker.class);
        when(context.getBroker()).thenReturn(broker);
    }

    @Test
    public void shouldHandleMessageWithAddHandlerPattern() throws ReflectiveOperationException, UnknownMessageFormatException {
        var handlerName = "AnHandler";
        when(context.getOutputStream()).thenReturn(mock(OutputStream.class));
        AddHandlerMessageHandler handler = new AddHandlerMessageHandler(context);
        handler.handleMessage("CONTROL.ADD_HANDLER " + handlerName);
        verify(context.getBroker()).addHandler(handlerName);
    }

    @Test(expected = UnknownMessageFormatException.class)
    public void shouldThrowWhenMessageIsUnknown() throws ReflectiveOperationException, UnknownMessageFormatException {
        var handlerName = "AnHandler";
        AddHandlerMessageHandler handler = new AddHandlerMessageHandler(context);
        handler.handleMessage("BAD " + handlerName);
        verify(context.getBroker()).addHandler(handlerName);
    }
}