package fr.gasser.daoexample.dao.connection;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.DriverManager;
import java.sql.SQLException;

public class SQLiteConnectionWrapper implements ConnectionWrapper {
    private static final Logger LOGGER = LoggerFactory.getLogger(SQLiteConnectionWrapper.class);

    private final String dbpath;
    private java.sql.Connection connection = null;

    private SQLiteConnectionWrapper(String dBPath) {
        dbpath = dBPath;
    }

    public SQLiteConnectionWrapper() {
        this(":memory:");
    }

    @Override
    public void connect() throws SQLiteConnectionException {
        try {
            Class.forName("org.sqlite.JDBC");
            connection = DriverManager.getConnection("jdbc:sqlite:" + dbpath);
            LOGGER.info("Connected to {} successfully.", dbpath);
        } catch (ClassNotFoundException | SQLException e) {
            throw new SQLiteConnectionException(e.getMessage());
        }
    }

    @Override
    public void close() {
        try {
            connection.close();
        } catch (SQLException e) {
            LOGGER.error("Unable to close the connection.", e);
        }
    }

    @Override
    public java.sql.Connection getConnection() {
        return connection;
    }
}
